@@ -250,12 +250,18 @@ void bpc_logMsgGet(char **mesg, size_t *mesgLen);
 void bpc_logMsgErrorCntGet(unsigned long *errorCnt);
 void bpc_logMsgCBSet(void (*cb)(int errFlag, char *mesg, size_t mesgLen));
 
+/*
+ * Compute file digest
+ */
+int bpc_fileDigest(char *fileName, int compress, bpc_digest *digest);
+
 /*
  * Directory operations
  */
 int bpc_path_create(char *path);
 int bpc_path_remove(bpc_deltaCount_info *deltaInfo, char *path, int compress);
 int bpc_path_refCountAll(bpc_deltaCount_info *deltaInfo, char *path, int compress, int incr);
+int bpc_path_refCountAllInodeMax(bpc_deltaCount_info *deltaInfo, char *path, int compress, int incr, unsigned int *inodeMax);
 int bpc_lockRangeFd(int fd, OFF_T offset, OFF_T len, int block);
 int bpc_unlockRangeFd(int fd, OFF_T offset, OFF_T len);
 int bpc_lockRangeFile(char *lockFile, OFF_T offset, OFF_T len, int block);
@@ -283,8 +289,8 @@ typedef struct {
      */
     ushort isTemp;
     uint32 mode;
-    uint32 uid;
-    uint32 gid;
+    uid_t uid;
+    gid_t gid;
     uint32 nlinks;
     time_t mtime;
     OFF_T size;
@@ -303,6 +309,7 @@ typedef struct {
 typedef struct {
     bpc_digest digest;
     ushort compress;
+    ushort needRewrite;
     /*
      * hash table of bpc_attrib_file entries, indexed by file name
      */
@@ -319,6 +326,7 @@ size_t bpc_attrib_xattrList(bpc_attrib_file *file, char *list, size_t listLen, i
 void bpc_attrib_fileInit(bpc_attrib_file *file, char *fileName, int xattrNumEntries);
 void bpc_attrib_fileDestroy(bpc_attrib_file *file);
 bpc_attrib_file *bpc_attrib_fileGet(bpc_attrib_dir *dir, char *fileName, int allocate_if_missing);
+int bpc_attrib_fileIterate(bpc_attrib_dir *dir, bpc_attrib_file **file, uint *idx);
 void bpc_attrib_fileCopyOpt(bpc_attrib_file *fileDest, bpc_attrib_file *fileSrc, int overwriteEmptyDigest);
 void bpc_attrib_fileCopy(bpc_attrib_file *fileDest, bpc_attrib_file *fileSrc);
 int bpc_attrib_fileCompare(bpc_attrib_file *file0, bpc_attrib_file *file1);
@@ -327,11 +335,13 @@ int bpc_attrib_fileCount(bpc_attrib_dir *dir);
 char *bpc_attrib_fileType2Text(int type);
 void bpc_attrib_dirInit(bpc_attrib_dir *dir, int compressLevel);
 void bpc_attrib_dirDestroy(bpc_attrib_dir *dir);
+int bpc_attrib_dirNeedRewrite(bpc_attrib_dir *dir);
 ssize_t bpc_attrib_getEntries(bpc_attrib_dir *dir, char *entries, ssize_t entrySize);
 void bpc_attrib_dirRefCount(bpc_deltaCount_info *deltaInfo, bpc_attrib_dir *dir, int incr);
+void bpc_attrib_dirRefCountInodeMax(bpc_deltaCount_info *deltaInfo, bpc_attrib_dir *dir, int incr, unsigned int *inodeMax);
 void bpc_attrib_attribFilePath(char *path, char *dir, char *attribFileName);
 bpc_digest *bpc_attrib_dirDigestGet(bpc_attrib_dir *dir);
-uchar *bpc_attrib_buf2file(bpc_attrib_file *file, uchar *buf, uchar *bufEnd, int xattrNumEntries);
+uchar *bpc_attrib_buf2file(bpc_attrib_file *file, uchar *buf, uchar *bufEnd, int xattrNumEntries, int *xattrFixup);
 uchar *bpc_attrib_buf2fileFull(bpc_attrib_file *file, uchar *buf, uchar *bufEnd);
 uchar *bpc_attrib_file2buf(bpc_attrib_file *file, uchar *buf, uchar *bufEnd);
 int bpc_attrib_digestRead(bpc_attrib_dir *dir, bpc_digest *digest, char *attribPath);
