@@ -63,7 +63,7 @@ void bpc_attrib_backwardCompat(int writeOldStyleAttribFile, int keepOldAttribFil
 {
     if ( writeOldStyleAttribFile >= 0 ) WriteOldStyleAttribFile = writeOldStyleAttribFile;
     if ( keepOldAttribFiles >= 0 )      KeepOldAttribFiles = keepOldAttribFiles;
-    if ( BPC_LogLevel >= 2 ) {
+    if ( BPC_LogLevel >= 5 ) {
         bpc_logMsgf("bpc_attrib_backwardCompat: WriteOldStyleAttribFile = %d, KeepOldAttribFiles = %d\n",
                      WriteOldStyleAttribFile, KeepOldAttribFiles);
     }
@@ -121,7 +121,23 @@ int bpc_attrib_xattrDeleteAll(bpc_attrib_file *file)
  */
 int bpc_attrib_xattrSetValue(bpc_attrib_file *file, void *key, int keyLen, void *value, uint32 valueLen)
 {
-    bpc_attrib_xattr *xattr = bpc_attrib_xattrGet(file, key, keyLen, 1);
+    bpc_attrib_xattr *xattr;
+    char keyCopy[BPC_MAXPATHLEN];
+
+    if ( ((char*)key)[keyLen - 1] != 0x0 ) {
+        if ( keyLen >= BPC_MAXPATHLEN - 8 ) {
+            bpc_logMsgf("bpc_attrib_xattrSetValue: BOTCH: key not 0x0 terminated; too long to repair (keyLen %u)\n", keyLen);
+            return -1;
+        } else {
+            memcpy(keyCopy, key, keyLen);
+            keyCopy[keyLen++] = 0x0;
+            key = keyCopy;
+            if ( BPC_LogLevel >= 6 ) {
+                bpc_logMsgf("bpc_attrib_xattrSetValue: fixup: appended 0x0 to xattr name '%s' (keyLen now %u)\n", (char*)key, keyLen);
+            }
+        }
+    }
+    xattr = bpc_attrib_xattrGet(file, key, keyLen, 1);
 
     if ( !xattr->value ) {
         /*
@@ -223,9 +239,14 @@ static void bpc_attrib_xattrListKey(bpc_attrib_xattr *xattr, xattrList_info *inf
             return;
         }
         /*
-         * keyLen already includes the \0 terminating byte
+         * confirm that keyLen includes the \0 terminating byte
          */
         memcpy(info->list + info->idx, xattr->key.key, xattr->key.keyLen);
+        if ( xattr->key.keyLen >= 1 && info->list[info->idx + xattr->key.keyLen - 1] != 0x0 ) {
+            info->list[info->idx + xattr->key.keyLen - 1] = 0x0;
+            bpc_logMsgf("bpc_attrib_xattrListKey: BOTCH: truncated xattr name '%s' to match keyLen %u\n", info->list + info->idx, xattr->key.keyLen);
+        }
+        if ( BPC_LogLevel >= 6 ) bpc_logMsgf("bpc_attrib_xattrListKey: adding %s\n", info->list + info->idx);
         info->idx += xattr->key.keyLen;
     } else {
         info->idx += xattr->key.keyLen;
@@ -369,6 +390,13 @@ void bpc_attrib_fileDeleteName(bpc_attrib_dir *dir, char *fileName)
     bpc_hashtable_nodeDelete(&dir->filesHT, file);
 }
 
+int bpc_attrib_fileIterate(bpc_attrib_dir *dir, bpc_attrib_file **file, uint *idx)
+{
+    *file = bpc_hashtable_nextEntry(&dir->filesHT, idx);
+    if ( !*file ) return -1;
+    return 0;
+}
+
 int bpc_attrib_fileCount(bpc_attrib_dir *dir)
 {
     return bpc_hashtable_entryCount(&dir->filesHT);
@@ -384,6 +412,7 @@ void bpc_attrib_dirInit(bpc_attrib_dir *dir, int compressLevel)
 {
     dir->digest.len = 0;
     dir->compress = compressLevel;
+    dir->needRewrite = 0;
     bpc_hashtable_create(&dir->filesHT, 512, sizeof(bpc_attrib_file));
 }
 
@@ -393,6 +422,11 @@ void bpc_attrib_dirDestroy(bpc_attrib_dir *dir)
     bpc_hashtable_destroy(&dir->filesHT);
 }
 
+int bpc_attrib_dirNeedRewrite(bpc_attrib_dir *dir)
+{
+    return dir->needRewrite;
+}
+
 typedef struct {
     bpc_deltaCount_info *deltaInfo;
     int incr;
@@ -566,21 +600,19 @@ static int64 getVarInt_v3(uchar **bufPP, uchar *bufEnd)
 static void setVarInt(uchar **bufPP, uchar *bufEnd, int64 value)
 {
     uchar *bufP = *bufPP;
+    int maxBytes = (sizeof(value) * 8 + 6) / 7;
 
-    if ( value < 0 ) {
-        bpc_logErrf("setVarInt botch: got negative argument %ld; setting to 0\n", (long int)value);
-        value = 0;
-    }
     do {
         uchar c = value & 0x7f;
         value >>= 7;
-        if ( value ) c |= 0x80;
+        maxBytes--;
+        if ( value && maxBytes > 0 ) c |= 0x80;
         if ( bufP < bufEnd ) {
             *bufP++ = c;
         } else {
             bufP++;
         }
-    } while ( value );
+    } while ( value && maxBytes > 0 );
     *bufPP = bufP;
 }
 
@@ -591,7 +623,7 @@ static void setVarInt(uchar **bufPP, uchar *bufEnd, int64 value)
  * If there isn't enough data to extract a complete file structure, the return value
  * will be greater than bufEnd.  You should gather more data and re-call the function.
  */
-uchar *bpc_attrib_buf2file(bpc_attrib_file *file, uchar *buf, uchar *bufEnd, int xattrNumEntries)
+uchar *bpc_attrib_buf2file(bpc_attrib_file *file, uchar *buf, uchar *bufEnd, int xattrNumEntries, int *xattrFixup)
 {
     uchar *bufP   = buf;
     int i;
@@ -618,6 +650,9 @@ uchar *bpc_attrib_buf2file(bpc_attrib_file *file, uchar *buf, uchar *bufEnd, int
         uint valueLen = getVarInt(&bufP, bufEnd);
 
         if ( bufP + keyLen + valueLen <= bufEnd ) {
+            if ( xattrFixup && bufP[keyLen - 1] != 0x0 ) {
+                *xattrFixup = 1;
+            }
             bpc_attrib_xattrSetValue(file, bufP, keyLen, bufP + keyLen, valueLen);
         }
         bufP += keyLen + valueLen;
@@ -644,8 +679,10 @@ uchar *bpc_attrib_buf2fileFull(bpc_attrib_file *file, uchar *bufP, uchar *bufEnd
         return NULL;
     }
     bufP += fileNameLen;
+    bpc_attrib_xattrDeleteAll(file);
     xattrNumEntries = getVarInt(&bufP, bufEnd);
-    bufP = bpc_attrib_buf2file(file, bufP, bufEnd, xattrNumEntries);
+    if ( BPC_LogLevel >= 6 ) bpc_logMsgf("bpc_attrib_buf2fileFull: xattrNumEntries = %d\n", xattrNumEntries);
+    bufP = bpc_attrib_buf2file(file, bufP, bufEnd, xattrNumEntries, NULL);
     return bufP;
 }
 
@@ -849,6 +886,7 @@ int bpc_attrib_dirRead(bpc_attrib_dir *dir, char *dirPath, char *attribFilePath,
             char *fileName;
             bpc_attrib_file *file;
             uchar *bufPsave = bufP;
+            int xattrFixup = 0;
 
             if ( nRead == sizeof(buf) && bufP > buf + nRead - 2 * BPC_MAXPATHLEN
                     && read_more_data(&fd, buf, sizeof(buf), &nRead, &bufP, attribPath) ) {
@@ -877,7 +915,8 @@ int bpc_attrib_dirRead(bpc_attrib_dir *dir, char *dirPath, char *attribFilePath,
             bpc_attrib_fileInit(file, fileName, xattrNumEntries);
             file->backupNum = backupNum;
 
-            bufP = bpc_attrib_buf2file(file, bufP, buf + nRead, xattrNumEntries);
+            bufP = bpc_attrib_buf2file(file, bufP, buf + nRead, xattrNumEntries, &xattrFixup);
+            dir->needRewrite |= xattrFixup;
             if ( bufP > buf + nRead ) {
                 /*
                  * Need to get more data and try again.  We have allocated file->name,
@@ -961,6 +1000,7 @@ int bpc_attrib_dirRead(bpc_attrib_dir *dir, char *dirPath, char *attribFilePath,
 typedef struct {
     uchar *bufP;
     uchar *bufEnd;
+    uint numEntries;
 } buf_info;
 
 typedef struct {
@@ -983,8 +1023,12 @@ static void bpc_attrib_xattrWrite(bpc_attrib_xattr *xattr, buf_info *info)
     setVarInt(&info->bufP, info->bufEnd, xattr->key.keyLen);
     setVarInt(&info->bufP, info->bufEnd, xattr->valueLen);
 
-    if ( info->bufP + xattr->key.keyLen <= info->bufEnd ) {
+    if ( xattr->key.keyLen >= 1 && info->bufP + xattr->key.keyLen <= info->bufEnd ) {
         memcpy(info->bufP, xattr->key.key, xattr->key.keyLen);
+        if ( info->bufP[xattr->key.keyLen - 1] != 0x0 ) {
+            info->bufP[xattr->key.keyLen - 1] = 0x0;
+            bpc_logMsgf("bpc_attrib_xattrWrite: BOTCH: truncated xattr name '%s' to match keyLen %u\n", info->bufP, xattr->key.keyLen);
+        }
     }
     info->bufP += xattr->key.keyLen;
 
@@ -992,6 +1036,7 @@ static void bpc_attrib_xattrWrite(bpc_attrib_xattr *xattr, buf_info *info)
         memcpy(info->bufP, xattr->value, xattr->valueLen);
     }
     info->bufP += xattr->valueLen;
+    info->numEntries++;
 }
 
 /*
@@ -1031,10 +1076,13 @@ uchar *bpc_attrib_file2buf(bpc_attrib_file *file, uchar *buf, uchar *bufEnd)
     }
     bufP += file->digest.len;
 
-    info.bufEnd = bufEnd;
-    info.bufP   = bufP;
+    info.bufEnd     = bufEnd;
+    info.bufP       = bufP;
+    info.numEntries = 0;
     bpc_hashtable_iterate(&file->xattrHT, (void*)bpc_attrib_xattrWrite, &info);
-
+    if ( info.numEntries != xattrEntryCnt ) {
+        bpc_logErrf("bpc_attrib_file2buf: BOTCH: wrote %u xattr entries vs %u; attrib file corrupted\n", info.numEntries, xattrEntryCnt);
+    }
     return info.bufP;
 }
 
@@ -1180,6 +1228,7 @@ int bpc_attrib_dirWrite(bpc_deltaCount_info *deltaInfo, bpc_attrib_dir *dir, cha
     char *p;
     int fdNum;
     size_t attribPathLen, baseAttribFileNameLen;
+    int digestChanged;
 
     if ( WriteOldStyleAttribFile ) return bpc_attrib_dirWriteOld(deltaInfo, dir, dirPath, attribFileName, oldDigest);
 
@@ -1232,34 +1281,39 @@ int bpc_attrib_dirWrite(bpc_deltaCount_info *deltaInfo, bpc_attrib_dir *dir, cha
         }
         attribPath[attribPathLen++] = '_';
         bpc_digest_digest2str(&digest, attribPath + attribPathLen);
+        /*
+         * Now create an empty attrib file with the file name digest
+         */
+        if ( (fdNum = open(attribPathTemp, O_WRONLY | O_CREAT | O_TRUNC, 0660)) < 0 ) {
+            bpc_logErrf("bpc_attrib_dirWrite: can't open/create raw %s for writing\n", attribPathTemp);
+            return -1;
+        }
+        close(fdNum);
+        if ( rename(attribPathTemp, attribPath) ) {
+            bpc_logErrf("bpc_attrib_dirWrite: rename from %s to %s failed\n", attribPathTemp, attribPath);
+            return -1;
+        }
+        if ( BPC_LogLevel >= 5 ) bpc_logMsgf("bpc_attrib_dirWrite: created new attrib file %s\n", attribPath);
     } else {
-        digest.len = 0;
+        memset(&digest, 0, sizeof(digest));
         attribPath[attribPathLen++] = '_';
         strcpy(attribPath + attribPathLen, "0");
+        if ( BPC_LogLevel >= 5 ) bpc_logMsgf("bpc_attrib_dirWrite: skipping creating new empty attrib file %s\n", attribPath);
+        unlink(attribPath);
     }
     
-    /*
-     * Now create an empty attrib file
-     */
-    if ( (fdNum = open(attribPathTemp, O_WRONLY | O_CREAT | O_TRUNC, 0660)) < 0 ) {
-        bpc_logErrf("bpc_attrib_dirWrite: can't open/create raw %s for writing\n", attribPathTemp);
-        return -1;
-    }
-    close(fdNum);
-    if ( rename(attribPathTemp, attribPath) ) {
-        bpc_logErrf("bpc_attrib_dirWrite: rename from %s to %s failed\n", attribPathTemp, attribPath);
-        return -1;
-    }
-    if ( BPC_LogLevel >= 5 ) bpc_logMsgf("bpc_attrib_dirWrite: created new attrib file %s\n", attribPath);
-
     if ( BPC_LogLevel >= 8 ) bpc_logMsgf("bpc_attrib_dirWrite: new attrib digest = 0x%02x%02x%02x..., oldDigest = 0x%02x%02x...\n",
                 digest.digest[0], digest.digest[1], digest.digest[2],
                 oldDigest ? oldDigest->digest[0] : 0x0, oldDigest ? oldDigest->digest[1] : 0x0);
-    bpc_poolRefDeltaUpdate(deltaInfo, dir->compress, &digest, 1);
+
+    digestChanged = !oldDigest || bpc_digest_compare(&digest, oldDigest);
+    if ( digestChanged && digest.len > 0 ) {
+        bpc_poolRefDeltaUpdate(deltaInfo, dir->compress, &digest, 1);
+    }
 
     if ( oldDigest ) {
-        if ( !bpc_digest_compare(&digest, oldDigest) ) {
-            if ( BPC_LogLevel >= 2 ) bpc_logMsgf("bpc_attrib_dirWrite: old attrib has same digest; no changes to ref counts\n");
+        if ( !digestChanged ) {
+            if ( BPC_LogLevel >= 4 ) bpc_logMsgf("bpc_attrib_dirWrite: old attrib has same digest; no changes to ref counts\n");
             return 0;
         }
         if ( attribPathLen + oldDigest->len * 2 + 2 >= sizeof(attribPath) ) {
